@@ -1,22 +1,16 @@
-#![allow(unused_imports, dead_code)]
-use log::{info, error, debug};
+#[allow(unused_imports)]
+use tracing::{info, warn, debug, error, trace, instrument, span, Level};
 use ate::{compact::CompactMode, prelude::*};
-use std::env;
-use std::io::ErrorKind;
-use directories::BaseDirs;
-use std::sync::Arc;
-use std::ops::Deref;
 use std::time::Duration;
 use url::Url;
-use tokio::select;
 
-use clap::Clap;
+use clap::Parser;
 
 mod flow;
 
 use crate::flow::ChainFlow;
 
-#[derive(Clap)]
+#[derive(Parser)]
 #[clap(version = "1.4", author = "John S. <johnathan.sharratt@gmail.com>")]
 struct Opts {
     /// Sets the level of log verbosity, can be used multiple times
@@ -26,8 +20,8 @@ struct Opts {
     /// Logs debug info to the console
     #[clap(short, long)]
     debug: bool,
-    /// URL where the user is authenticated (default port=5001)
-    #[clap(short, long, default_value = "tcp://auth.tokera.com/")]
+    /// URL where the user is authenticated
+    #[clap(short, long, default_value = "ws://tokera.com/auth")]
     auth: Url,
     /// Indicates no authentication server will be used meaning all new chains
     /// created by clients allow anyone to write new root nodes.
@@ -42,7 +36,7 @@ struct Opts {
     /// Disbles wire encryption which would otherwise be turned on when running in 'centralized' mode.
     #[clap(long)]
     no_wire_encryption: bool,
-    /// Trust mode that the database server will run under - valid values are either
+    /// Trust mode that the datachain server will run under - valid values are either
     /// 'distributed' or 'centralized'. When running in 'distributed' mode the
     /// server itself does not need to be trusted in order to trust the data it holds
     /// however it has a significant performance impact on write operations while the
@@ -56,27 +50,35 @@ struct Opts {
     /// Address that DNS queries will be sent to
     #[clap(long, default_value = "8.8.8.8")]
     dns_server: String,
+
     #[clap(subcommand)]
     subcmd: SubCommand,
 }
 
-#[derive(Clap)]
+#[derive(Parser)]
 enum SubCommand {
     #[clap()]
     Solo(Solo),
 }
-/// Runs a solo ATE database and listens for connections from clients
-#[derive(Clap)]
+/// Runs a solo ATE datachain and listens for connections from clients
+#[derive(Parser)]
 struct Solo {
     /// Path to the log files where all the file system data is stored
     #[clap(index = 1, default_value = "/opt/ate")]
     logs_path: String,
-    /// IP address that the database server will isten on
-    #[clap(short, long, default_value = "0.0.0.0")]
-    listen: String,
-    /// Port that the database server will listen on
-    #[clap(short, long, default_value = "5000")]
-    port: u16,
+    /// Path to the backup and restore location of log files
+    #[clap(short, long)]
+    backup_path: Option<String>,
+    /// Address that the datachain server(s) are listening and that
+    /// this server can connect to if the chain is on another mesh node
+    #[clap(short, long, default_value = "ws://localhost:5000/db")]
+    url: url::Url,
+    /// IP address that the datachain server will isten on
+    #[clap(short, long, default_value = "::")]
+    listen: IpAddr,
+    /// Ensures that this datachain runs as a specific node_id
+    #[clap(short, long)]
+    node_id: Option<u32>,
     /// Mode that the compaction will run under (valid modes are 'never', 'modified', 'timer', 'factor', 'size', 'factor-or-timer', 'size-or-timer')
     #[clap(long, default_value = "factor-or-timer")]
     compact_mode: CompactMode,
@@ -93,32 +95,30 @@ struct Solo {
 
 fn ctrl_channel() -> tokio::sync::watch::Receiver<bool> {
     let (sender, receiver) = tokio::sync::watch::channel(false);
-    ctrlc::set_handler(move || {
+    ctrlc_async::set_handler(move || {
         let _ = sender.send(true);
     }).unwrap();
     receiver
 }
 
-#[tokio::main]
+#[tokio::main(flavor = "multi_thread")]
 async fn main() -> Result<(), AteError> {
     let opts: Opts = Opts::parse();
     //let opts = main_debug();
 
-    let mut log_level = match opts.verbose {
-        0 => "error",
-        1 => "warn",
-        2 => "info",
-        _ => "debug",
-    };
-    if opts.debug { log_level = "debug"; }
-    env_logger::Builder::from_env(env_logger::Env::default().default_filter_or(log_level)).init();
+    ate::log_init(opts.verbose, opts.debug);
 
     let wire_encryption = match opts.wire_encryption {
         Some(a) => Some(a),
         None => {
-            match opts.trust {
-                TrustMode::Centralized => Some(KeySize::Bit128),
-                TrustMode::Distributed => None
+            match opts.trust.is_centralized() {
+                true => {
+                    match opts.no_wire_encryption {
+                        false => Some(KeySize::Bit128),
+                        true => None,
+                    }
+                },
+                false => None
             }
         }
     };
@@ -126,16 +126,15 @@ async fn main() -> Result<(), AteError> {
     let mut conf = AteConfig::default();
     conf.dns_sec = opts.dns_sec;
     conf.dns_server = opts.dns_server;
-    conf.wire_encryption = wire_encryption;
 
     let auth = match opts.no_auth {
-        false if opts.trust == TrustMode::Centralized => Some(opts.auth),
+        false if opts.trust.is_centralized() => Some(opts.auth),
         _ => None,
     };
     
     match opts.subcmd {
         SubCommand::Solo(solo) => {
-            main_solo(solo, conf, auth, opts.trust).await?;
+            main_solo(solo, conf, auth, opts.trust, wire_encryption).await?;
         }
     }
 
@@ -144,21 +143,26 @@ async fn main() -> Result<(), AteError> {
     Ok(())
 }
 
-async fn main_solo(solo: Solo, mut cfg_ate: ConfAte, auth: Option<url::Url>, trust: TrustMode) -> Result<(), AteError>
+async fn main_solo(solo: Solo, mut cfg_ate: ConfAte, auth: Option<url::Url>, trust: TrustMode, wire_encryption: Option<KeySize>) -> Result<(), AteError>
 {
     // Create the chain flow and generate configuration
     cfg_ate.log_path = Some(shellexpand::tilde(&solo.logs_path).to_string());
+    cfg_ate.backup_path = solo.backup_path.as_ref().map(|a| shellexpand::tilde(a).to_string());
     cfg_ate.compact_mode = solo.compact_mode
         .with_growth_factor(solo.compact_threshold_factor)
         .with_growth_size(solo.compact_threshold_size)
         .with_timer_value(Duration::from_secs(solo.compact_timer));
-
+    
     // Create the chain flow and generate configuration
-    let flow = ChainFlow::new(&cfg_ate, auth, trust).await;
+    let flow = ChainFlow::new(&cfg_ate, auth, solo.url.clone(), trust).await;
 
-    // Create the server and listen on port 5000
-    let cfg_mesh = ConfMesh::solo(solo.listen.as_str(), solo.port);
-    let _server = create_server(&cfg_ate, &cfg_mesh, Box::new(flow)).await;
+    // Create the server and listen on the port
+    let mut cfg_mesh = ConfMesh::solo_from_url(&cfg_ate, &solo.url, &solo.listen, None, solo.node_id).await?;
+    cfg_mesh.wire_protocol = StreamProtocol::parse(&solo.url)?;
+    cfg_mesh.wire_encryption = wire_encryption;
+
+    let server = create_server(&cfg_mesh).await?;
+    server.add_route(Box::new(flow), &cfg_ate).await?;
 
     // Wait for ctrl-c
     eprintln!("Press ctrl-c to exit");
@@ -166,6 +170,8 @@ async fn main_solo(solo: Solo, mut cfg_ate: ConfAte, auth: Option<url::Url>, tru
     while *exit.borrow() == false {
         exit.changed().await.unwrap();
     }
+    println!("Shutting down...");
+    server.shutdown().await;
     println!("Goodbye!");
     Ok(())
 }
\ No newline at end of file
